<?php
class GO_Cpm
{

	var $id_base;
	var $post_meta_key;

	function __construct( $cpm_name = FALSE )
	{
		// set base IDs and such
		$this->id_base = = $this->post_meta_key = $cpm_name;

		// filter post saves to set postmeta from metabox entries
		add_action( 'save_post', array( $this , '_save_post' ));
	}

	/**
	 * The metabox callback function
	 *
	 * This function is inserts the nonce field and calls the child class' metabox() method to display the form inside the metabox
	 *
	 */
	function _metabox( $post )
	{
		$this->nonce_field();
		$this->metabox( $post , $this->get_meta( $post->ID ));
	}

	/**
	 * Inserts a nonce in the metabox
	 *
	 * This function is called internally before the child class' metabox() method is called
	 *
	 * @uses wp_nonce_field()
	 *
	 */
	function nonce_field()
	{
		wp_nonce_field( $this->id_base , $this->id_base .'-nonce' );
	}

	/**
	 * Verifies the nonce
	 *
	 * This function is called internally before the child class' update() method is called
	 *
	 * @uses wp_verify_nonce()
	 *
	 */
	function verify_nonce()
	{
		return wp_verify_nonce( $_POST[ $this->id_base .'-nonce' ] , $this->id_base );
	}


	/**
	 * Constructs name attributes for use in metabox() fields
	 *
	 * This function should be used in metabox() methods to create name attributes for fields to be saved by update()
	 *
	 * @param string $field_name Field name
	 * @return string Name attribute for $field_name
	 */
	function get_field_name( $field_name )
	{
		return $this->id_base . '[' . $field_name . ']';
	}

	/**
	 * Constructs id attributes for use in metabox() fields
	 *
	 * This function should be used in metabox() methods to create id attributes for fields to be saved by update()
	 *
	 * @param string $field_name Field name
	 * @return string ID attribute for $field_name
	 */
	function get_field_id( $field_name )
	{
		return $this->id_base . '-' . $field_name;
	}

	/**
	 * Retrieve metadata for the specified object.
	 *
	 * @uses get_post_meta()
	 *
	 * @param int $object_id ID of the object metadata is for
	 * @return string|array Single metadata value, or array of values
	 */
	function get_meta( $post_id )
	{
		return get_post_meta( $post_id , $this->post_meta_key , TRUE );
	}

	/**
	 * Update metadata for the specified object. Calls the child class' update() method to sanitize the data. 
	 * If no meta value exists for the specified object ID, the metadata will be added.
	 *
	 * @uses update_post_meta()
	 *
	 * @param int $object_id ID of the object metadata is for
	 * @param string $meta_value Metadata value
	 * @return bool True on successful update, false on failure.
	 */
	function update_meta( $post_id , $meta )
	{
		// make sure meta is added to the post, not a revision
		if ( $_post_id = wp_is_post_revision( $post_id ))
			$post_id = $_post_id;

		// apply sanitization from the child class if we have an update method defined
		if( is_callable( array( $this , 'update' )))
		{
			$meta = $this->update( $meta , $this->get_meta( $post_id ));
		}

		update_post_meta( $post_id , $this->post_meta_key , $meta );
	}
}